'
' Template for user code replacement of the compiler's bit-bashed I2C BusIn/BusOut library subroutines
' The exact mechanism used by the compiler is reproduced using BASIC commands
'
Reminders = Off
  'Optimiser_Level = 3                 ' Uncomment for maximum optimisation
'
' Disable the compiler's Busin and Busout commands
'
  #disable BusOut, Busoutb, BusIn, Businb, BStart, BusNack
'
' Declare the Pins used for bit-bashed I2C communications
'
  Declare SDA_Pin = PORTC.4            ' SDA connected to thisd pin
  Declare SCL_Pin = PORTC.3            ' SCL connected to this pin
'
' Create some system variables for the routines
'
  Dim BPF As Byte System               ' Holds state flags
  Dim GPR As Byte System               ' General purpose
  Dim PP1 As Byte System               ' Output from Busin
  Dim PP1H As Byte System              ' Used for loop counting
  Dim PP3 As Byte System               ' General purpose
  Dim PP4 As Byte System               ' Bit-0 returns the received Ack state
'
' I2C Flags
'
  Symbol tSkipAckAfterRead = BPF.2     ' I2C Skip Acknowledge after read If Set
  Symbol tReadWriteThenStop = BPF.4    ' I2C R/W Then Stop flag
  Symbol tSendStartBeforeRead = BPF.5  ' I2C reading flag. When Set, a Start, Command sequence is first initiated before a Byte is read
  Symbol tStartSent = BPF.6            ' I2C Start condition sent flag
  Symbol tBusyWriting = BPF.7          ' I2C writing flag

'-------------------------------------------------------------------------------------------------
  GoTo Over_BusinBusout_Subs           ' Jump over the subroutines
'-------------------------------------------------------------------------------------------------
' Busin
' Receive I2C data
'
' Input     : None
' Output    : PP1 or WREG
'           : Carry Flag or PP4.0 indicate ACK return
' Notes     : Do not disturb bits 0 or 1 of BPF as the compiler uses these elsewhere
'           : Call businb to read a byte without sending an Acknowledge or Stop condition
'           : Call busin2 to read a byte and send an Acknowledge and Stop condition after it
'
#ifdef BusInb#Req
@BusIn#Req = 1
' Read a Byte from the bus without sending an Acknowledge
Businb:
  Clear tReadWriteThenStop                ' Don't send a Stop condition
  Set tSendStartBeforeRead                ' Indicate No Start condition and Command sent first
  Set tStartSent                          ' Not the first byte
  Set tBusyWriting                        ' No writing
  Set tSkipAckAfterRead                   ' Don't send Acknowledges condition after Writes
  GoTo Busin_GetByte                      ' Receive Data byte without a Stop condition, or an Acknowledge condition sent
#endif

#ifdef BusIn#Req
@BusOut#Req = 1
' Read a Byte from the bus without sending a Stop
Busin2:
  Set tReadWriteThenStop                  ' Indicate Send Stop condition after this byte
  GoTo BusinStopAfterByte                 ' Skip
' Read a Byte from the bus and send a Stop
@BusIn
  Clear tReadWriteThenStop                ' Indicate don't send Stop condition
BusinStopAfterByte:
  If tSendStartBeforeRead = 0 Then        ' Already reading?
    Set tSendStartBeforeRead              ' No. So indicate read
    GoSub Bus_SendStartAndCommand         ' Send Start condition and command
    Bc Bus_SendStop                       ' Jump if not Acknowledged?
  EndIf
  Clear tSkipAckAfterRead                 ' Indicate to Send acknowledges after writes
Busin_GetByte:
  PP1H = 8                                ' 8 bits to a byte, with no acknowledge
Busin_ReadLoop:
  GoSub Bus_GetBit                        ' Get a bit
  Rol PP1                                 ' Rotate it into place
  Djnz PP1H, Busin_ReadLoop               ' Do the next bit (if any)
  If tSkipAckAfterRead = 0 Then           ' Send an Acknowledge condition? (0 = yes , 1 = no)
    If tReadWriteThenStop = 0 Then        ' Yes. So Send a Stop condition?
      '
      ' Send an Acknowledge (Called when the BusAck command is used)
      '
@bus@ack
      GoSub Bus_DataLow                   ' Send an Acknowledge condition
    EndIf
    GoSub Bus_ClockHigh                   ' Toggle Clock
    GoSub Bus_ClockLow
    If tReadWriteThenStop = 1 Then        ' Send a Stop condition?
      GoSub Bus_SendStop                  ' Yes So send a Stop condition
    EndIf
  EndIf
  GoSub Bus_DataHigh                      ' Make sure Data line ends up as an Input
  Clear STATUSbits_C                      ' Set Carry for OK return
  WREG = PP1                              ' Place result in WREG before returning
  GoTo Bus_Exit                           ' Exit the routine
#endif
'-------------------------------------------------------------------------------------------------
' Busout
' Transmit I2C data
'
' Input     : WREG = Data/Command
'           : GPR = command (used internally)
' Output    : Carry Flag or PP4.0 indicate ACK return
' Notes     : Do not disturb bits 0 or 1 of BPF as the compiler uses these elsewhere
'           : Call busoutb if a Stop condition is not required after a byte is sent
'           : Call busout2 if a Stop condition is required after a byte is sent
'
#ifdef BusOutb#Req
@Busout#Req = 1
' Write a Byte to the bus without sending a Start and with Command first
Busoutb:
  Set tSendStartBeforeRead                ' Indicate No Start condition. And Command sent first
  Set tStartSent                          ' Indicate Not the first byte
  Set tBusyWriting                        ' Indicate No writing
  GoTo BOut                               ' Receive Data byte without a Stop condition sent
#endif

#ifdef BusOut#Req
' Write a Byte to the bus sending a Start first
Busout2:
  Set tReadWriteThenStop                  ' Indicate Stop condition after this byte
  GoTo BusoutStopAfterByte                ' Skip
BOut:
@BusOut
  Clear tReadWriteThenStop                ' Indicate Don't send Stop condition
BusoutStopAfterByte:
  If tStartSent = 0 Then                  ' Is this the first byte (command)
    GPR = WREG & 254                      ' Yes. So make sure R/W bit clear and save the command value for later
    Set tStartSent                        ' Indicate not the first byte anymore
    Clear STATUSbits_C                    ' Clear Carry for OK return
    GoTo Bus_Exit                         ' Exit the routine
  EndIf
  If tBusyWriting = 0 Then                ' Already writing?
    PP3 = WREG                            ' No. So save the byte
    Set tBusyWriting                      ' Indicate write
    GoSub Bus_SendStartAndCommand         ' Send Start condition and command
    If STATUSbits_C = 1 Then              ' Jump if not Acknowledged?
      GoTo Bus_SendStop
    EndIf
    WREG = PP3                            ' Get the byte back
  EndIf
  GoSub Bus_WriteByte                     ' Write a byte
  If STATUSbits_C = 0 Then                ' Stop if not Acknowledged?
    If tReadWriteThenStop = 0 Then        ' Send Stop condition?
      GoTo Bus_Exit                       ' No. So Exit
    EndIf
  EndIf
'
' Send a Stop Command (Called when the BStop command is used)
'
Bus_SendStop:
@bus@stp
  GoSub Bus_DataLow                       ' Start Data Low
  GoSub Bus_ClockHigh                     ' Set Clock High (Input)
  Clear tStartSent                        ' Indicate Reset first byte flag
  Clear tBusyWriting                      ' Indicate No longer writing
  Clear tSendStartBeforeRead              ' Indicate No longer reading either
  Clear tSkipAckAfterRead                 ' Indicate Send acknowledges after reads
  GoSub Bus_DataHigh                      ' Set Data High (Input)
'
' Exit the I2C subroutine
' Setting or Clearing PP4.0 and STATUS.C according to the ACK return
'
Bus_Exit:
  PP4.0 = STATUSbits_C
  Return
Bus_SendStartAndCommand:
  GoSub Bus_SendStart
  WREG = GPR                              ' Get the previously saved command value back
  WREG.0 = tSendStartBeforeRead           ' Add the Read bit (if required)
Bus_WriteByte:
  PP1 = WREG                              ' Save the Data
  PP1H = 8                                ' 8 bits to send
Bus_WriteLoop:
  Rol PP1                                 ' Get a bit to send into the Carry flag
  GoSub Bus_DataHigh                      ' Set bit High (Input)
  If STATUSbits_C = 0 Then                ' Skip if no bit
    GoSub Bus_DataLow                     ' Set bit Low
  EndIf
  GoSub Bus_ClockHigh                     ' Set Clock High (Input)
  GoSub Bus_ClockLow                      ' Pull Clock Low
  Djnz PP1H,Bus_WriteLoop                 ' Do the next bit (if any)
  GoSub Bus_DataHigh                      ' Make sure Data is High (Input)
'
' Get A Bit
'
Bus_GetBit:
  GoSub Bus_ClockHigh                     ' Set Clock High
  DelayCS 4                               ' Delay 4 instruction cycles
'
' STATUS.Carry bit reflects the SDA pin condition
'
  Clear STATUSbits_C                      ' Start with Carry clear
  If __SDA_PORT.__SDA_Pin = 1 Then        ' Skip the next if Data clear
    Set STATUSbits_C                      ' Otherwise, set carry
  EndIf
  GoSub Bus_ClockLow                      ' Pull Clock Low
  Return
'
' Send a Start/Restart Command (Called when the BStart/BResart commands are used)
'
Bus_SendStart:
@bus@strt
  GoSub Bus_DataHigh                      ' Make sure bus is idle
  GoSub Bus_ClockHigh
  GoSub Bus_DataLow                       ' Send Start condition
' Fall through to Clock Low
'
' Pin strobing routines
'
Bus_ClockLow:
  Low __SCL_PORT.__SCL_Pin                ' Pull Clock line Output Low
  GoTo Bus_Return                         ' Return via Slow Bus routine
Bus_ClockHigh:
  #ifdef Bus#SCL                          ' SCL_Hold declare used?
    High __SCL_PORT.__SCL_Pin             ' Set Clock line as Output High
    GoTo Bus_Return
  #else
    Input __SCL_PORT.__SCL_Pin            ' Set Clock line as Input
    GoTo Bus_Return                       ' Return via Slow Bus routine (If enabled)
  #endif
Bus_DataHigh:
  Input __SDA_PORT.__SDA_Pin              ' Set Data line as Input
  GoTo Bus_Return                         ' Return via Slow Bus routine (if enabled)
Bus_DataLow:
  Low __SDA_PORT.__SDA_Pin                ' Set Data line as Output Low
Bus_Return:
  #ifdef Slow#Bus                         ' Bus_Slow Declare used?
    DelayCS 12                            ' Delay for 12 cycles
  #endif
  Return
#endif
'-------------------------------------------------------------------------------------------------
' Send a Nack to the I2C bus
#ifdef BusNack#Req
@bus@nack
  Input __SDA_PORT.__SDA_Pin              ' Set Data line as Input
  #ifdef BUS#SCL                          ' SCL_Hold declare used?
    High __SCL_PORT.__SCL_Pin             ' Set Clock line as Output High
    #ifdef Slow#Bus                       ' Bus_Slow Declare used?
      DelayCS 12                          ' Delay for 12 cycles
    #endif
    Clear __SCL_PORT.__SCL_Pin            ' Set Clock line Low
  #else
    Input __SCL_PORT.__SCL_Pin            ' Set Clock line as Input
    #ifdef Slow#Bus                       ' Bus_Slow Declare used?
      DelayCS 12                          ' Delay for 12 cycles
    #endif
    Low __SCL_PORT.__SCL_Pin              ' Set Clock line Output Low
  #endif
  DelayCS 2                               ' A delay of 2 cycles
  Input __SCL_PORT.__SCL_Pin              ' Make sure Clock line ends as Input
  Return
#endif
'-------------------------------------------------------------------------------------------
Over_BusinBusout_Subs:



